// Socket for NEJE KZ laser cutter, containment powerbank and Raspberry Pi
// Homepage:
// HomoFaciens.de/technics-machines-Laser-Engraver-NEJE-KZ3000_ge.htm
$fn=100;

M3_hole = 3.5;
M4_nut_e = 7.66+1.0;

//Raspberry Pi mount dimensions
pi_m_l = 58;
pi_m_w = 49;
pi_m_shift_x = -48;
pi_m_shift_y = -15;
pi_m_d = 5;
pi_m_screw_d = 2;
pi_m_d_tolerance = 0.3;
pi_m_h = 6;
pi_m_socket = 1;

//Battery mount dimensions
bat_m_l = 70;
bat_m_w = 85;
bat_m_d = 10;
bat_m_h = 7;
bat_m_shift_x = 37;
bat_m_shift_y = -15;

// Socket dimensions
s_w = 162;
s_w2 = 150;
s_l = 150;
s_h = 50;
s_h2 = 35;
s_wall = 2;
screw_dis = 62;
screw_h = s_h - 5;
c_edge1 = 15;
c_edge2 = 8;
cut_w = 133;
cut_h = 5;


socket();
translate([0, 150, 0]){
  PowerBankMount();
}

module socket(){
  // Bottom plate
  translate([0, 0, s_wall/2]){
    difference(){
      union(){
        a=s_w+2*s_wall;
        b=s_l+2*s_wall;
        cube(size=[a, b, s_wall], center=true);
        translate([bat_m_shift_x, bat_m_shift_y, -s_wall/2]){
          translate([-bat_m_w/2, bat_m_l/2, bat_m_h/2]){
            cylinder(d=bat_m_d, h=bat_m_h, center=true);
          }
          translate([-bat_m_w/2, -bat_m_l/2, bat_m_h/2]){
            cylinder(d=bat_m_d, h=bat_m_h, center=true);
          }
        }
      }
      translate([bat_m_shift_x, bat_m_shift_y, -s_wall/2]){
        translate([-bat_m_w/2, bat_m_l/2, bat_m_h/2]){
          cylinder(d=M3_hole, h=bat_m_h+0.1, center=true);
        }
        translate([-bat_m_w/2, -bat_m_l/2, bat_m_h/2]){
          cylinder(d=M3_hole, h=bat_m_h+0.1, center=true);
        }
        translate([-bat_m_w/2, bat_m_l/2, s_wall/2]){
          cylinder(d=6.5, h=s_wall+0.1, center=true);
        }
        translate([-bat_m_w/2, -bat_m_l/2, s_wall/2]){
          cylinder(d=6.5, h=s_wall+0.1, center=true);
        }
        translate([-bat_m_w/2, bat_m_l/2, s_wall+2]){
          cylinder(d1=6.5, d2=M3_hole, h=4, center=true);
        }
        translate([-bat_m_w/2, -bat_m_l/2, s_wall+2]){
          cylinder(d1=6.5, d2=M3_hole, h=4, center=true);
        }
      }
    }
  }
  //Raspberry Pi mounts
  translate([pi_m_shift_x, pi_m_shift_y, 0]){
    translate([pi_m_w/2, pi_m_l/2, pi_m_h/2]){
      difference(){
        cylinder(d=pi_m_d, h=pi_m_h, center=true);
        cylinder(d=pi_m_screw_d, h=pi_m_h*1.1, center=true);
      }
    }
    translate([-pi_m_w/2, pi_m_l/2, pi_m_h/2]){
      difference(){
        cylinder(d=pi_m_d, h=pi_m_h, center=true);
        cylinder(d=pi_m_screw_d, h=pi_m_h*1.1, center=true);
      }
    }
    translate([-pi_m_w/2, -pi_m_l/2, pi_m_h/2]){
      difference(){
        cylinder(d=pi_m_d, h=pi_m_h, center=true);
        cylinder(d=pi_m_screw_d, h=pi_m_h*1.1, center=true);
      }
    }
    translate([pi_m_w/2, -pi_m_l/2, pi_m_h/2]){
      difference(){
        cylinder(d=pi_m_d, h=pi_m_h, center=true);
        cylinder(d=pi_m_screw_d, h=pi_m_h*1.1, center=true);
      }
    }
  }

  // Side walls
  difference(){
    union(){
      translate([(s_w+s_wall)/2, 0, s_h/2]){
        cube(size=[s_wall, s_l, s_h], center=true);
      }
      translate([-(s_w+s_wall)/2, 0, s_h/2]){
        cube(size=[s_wall, s_l, s_h], center=true);
      }
      // Spacer enforcements
      e_a=(s_w-s_w2)/2;
      translate([(s_w-e_a)/2, screw_dis/2, screw_h]){
        rotate([0, 90, 0]){
          enforcement(w1=7, w2=20, l=10, height=e_a);
        }
      }
      translate([(s_w-e_a)/2, -screw_dis/2, screw_h]){
        rotate([0, 90, 0]){
          enforcement(w1=7, w2=20, l=10, height=e_a);
        }
      }
      translate([-(s_w-e_a)/2, -screw_dis/2, screw_h]){
        rotate([0, 90, 180]){
          enforcement(w1=7, w2=20, l=10, height=e_a);
        }
      }
      translate([-(s_w-e_a)/2, screw_dis/2, screw_h]){
        rotate([0, 90, 180]){
          enforcement(w1=7, w2=20, l=10, height=e_a);
        }
      }

      translate([0, (s_l+s_wall)/2, s_h/2]){
        cube(size=[s_w+2*s_wall, s_wall, s_h], center=true);
      }
      translate([0, -(s_l+s_wall)/2, s_h/2]){
        cube(size=[s_w+2*s_wall, s_wall, s_h], center=true);
      }
      //corner points
      a=(s_w+s_wall-c_edge1)/2;
      b=(s_l+s_wall-c_edge2)/2;
      translate([a, b, s_h2/2]){
        cube(size=[c_edge1, c_edge2, s_h2], center=true);
      }    
      translate([-a, b, s_h2/2]){
        cube(size=[c_edge1, c_edge2, s_h2], center=true);
      }    
      translate([-a, -b, s_h2/2]){
        cube(size=[c_edge1, c_edge2, s_h2], center=true);
      }    
      translate([a, -b, s_h2/2]){
        cube(size=[c_edge1, c_edge2, s_h2], center=true);
      }    
    }
    // Mount holes
    translate([0, screw_dis/2, screw_h]){
      rotate([0, 90, 0]){
        long_hole(w=M3_hole, l=10, height=s_w*1.1);
      }
    }
    translate([0, -screw_dis/2, screw_h]){
      rotate([0, 90, 0]){
        long_hole(w=M3_hole, l=10, height=s_w*1.1);
      }
    }
    // Front cut
    translate([0, s_w/2, s_h/2+cut_h]){
      rotate([90, 0, 0]){
        roundedRect([cut_w, s_h, 10], 10, [1,1,0,0]);
      }
    }
  }
}


module PowerBankMount(){
  pb_l = 125;
  pb_w = 35;
  pb_wall = 2;
  //screw socket
  ss_h = 10;
  ss_d = 10;
  
  difference(){
    union(){
      roundedRect([pb_l, pb_w, pb_wall], 5);

      translate([(pb_l-ss_d)/2, -(pb_w-ss_d)/2, ss_h/2]){
        cylinder(d=10, h=ss_h, center=true);
      }
      translate([-(pb_l-ss_d)/2, -(pb_w-ss_d)/2, ss_h/2]){
        cylinder(d=10, h=ss_h, center=true);
      }
      
      translate([pb_l/2-25, pb_w/2-5, ss_h/2]){
        cylinder(d=10, h=ss_h, center=true);
      }
      translate([pb_l/2-25-bat_m_l, pb_w/2-5, ss_h/2]){
        cylinder(d=10, h=ss_h, center=true);
      }
    }
    translate([pb_l/2-25, pb_w/2-5, ss_h/2]){
      cylinder(d=M3_hole, h=30, center=true);
    }
    translate([pb_l/2-25-bat_m_l, pb_w/2-5, ss_h/2]){
      cylinder(d=M3_hole, h=30, center=true);
    }
  }
}

module enforcement(w1=6, w2=10, l=5, height=5){
  difference(){
    hull(){
      translate([l/2, 0, 0]){
        cylinder(d1=w1, d2=w2, h=height, center=true);
      }
      translate([-l/2, 0, 0]){
        cylinder(d1=w1, d2=w2, h=height, center=true);
      }
    }
    translate([-(l+w2)/2, 0, 0]){
      cube(size=[w2, w2, height*1.1], center=true);
    }
  }
}

module long_hole(w=M3_hole, l=5, height=5){
  hull(){
    translate([l/2, 0, 0]){
      cylinder(d=w, h=height, center=true);
    }
    translate([-l/2, 0, 0]){
      cylinder(d=w, h=height, center=true);
    }
  }
}

module roundedRect(size, radius, edges=[1, 1, 1, 1], vscale=[1,1], shift=0){
  x = size[0];
  y = size[1];
  z = size[2];

  translate([0, -shift*2, 0]){
    linear_extrude(height=z, scale=[vscale[0], vscale[1]]){
      translate([0, shift*2, 0]){
        hull(){
            // place 4 circles in the corners, with the given radius
            if(edges[0]){
              translate([(-x/2)+(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
            if(edges[1]){
              translate([(x/2)-(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[2]){
              translate([(-x/2)+(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[3]){
              translate([(x/2)-(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
          }
       }
    }
  }
}
