//Project: Mobile laser engraver with NEJE KZ3000 and Raspberry Pi
//Homepage: www.HomoFaciens.de/technics-machines-Laser-Engraver-NEJE-KZ3000_ge.htm
//Author Norbert Heinz
//Version: 1.0
//
// Compile with: gcc engrave.c -o engrave -lm

#include <stdint.h>
#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>




#define BAUDRATE B57600
#define USBPORT "/dev/ttyUSB0"
//#define USBPORT "/dev/serial"
//#define USBPORT "/dev/ttyACM0"
#define FALSE 0
#define TRUE 1


#define PI 3.1415927
// PORTPAUSE slows down USB communication
// Higher value means slower communication
// A too short pause time (in microseconds) troubles communication
#define PORTPAUSE 3000

int fd = 0;
struct termios TermOpt;
FILE *messageFile;




//+++++++++++++++++++++++ Start readport ++++++++++++++++++++++++++
int readport(void){
  int n;
  unsigned char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    printf("%02X ", buff);
    return buff;
  }
//  else{
//    printf("n=%d\n", n);
//  }
  usleep(PORTPAUSE);
  return -1;
}
//------------------------ End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport ++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;
  int BytesInBuffer;

  printf("%02X ", ValueToSend);
  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\n");
  }

 usleep(PORTPAUSE);
}
//------------------------ End sendport ----------------------------------


int openport(){

  fd=open(USBPORT, O_RDWR|O_NOCTTY|O_NDELAY);

  if(fd==-1){
      printf(" Cannot Open Port : %s \n",USBPORT);
      return 0;
  }

//  fcntl(fd,F_SETFL,0);

  if (tcgetattr(fd, &TermOpt) < 0) {
    printf("init_serialport: Couldn't get term attributes\n");
    return 0;
  }

  speed_t brate = BAUDRATE; // let you override switch below if needed

  cfsetispeed(&TermOpt,brate);// Set Baud Rate Of Input Stream .
  cfsetospeed(&TermOpt,brate);// Set Baud Rate Of Output Stream .

  TermOpt.c_cflag |= (CLOCAL | CREAD );// Options For Reading From Devices .
  TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);// Disable Canonical Input .
  TermOpt.c_oflag &= ~OPOST;// Disable Canonical Output .

  TermOpt.c_cc[VMIN]=1;// Options For Time Outs .
  TermOpt.c_cc[VTIME]=10;

  // 8N1
  TermOpt.c_cflag &= ~PARENB;   // turn off parity
  TermOpt.c_cflag &= ~CSTOPB;   // Clear stop field, only one stop bit used in communication (most common). If set, two stop bits used in communication
  TermOpt.c_cflag &= ~CSIZE;
  TermOpt.c_cflag |= CS8;       //Set to 8 bits per byte

  int RTS_flag, DTR_flag;
  RTS_flag = TIOCM_RTS;
  ioctl(fd,TIOCMBIS,&RTS_flag);//Set RTS pin
  DTR_flag = TIOCM_DTR;
  ioctl(fd,TIOCMBIS,&DTR_flag);//Set DTR pin


  tcsetattr(fd,TCSANOW,&TermOpt);// Update Changes .

  return 1;// Return Device Ptr .
}


//+++++++++++++++++++++++ Start openport ++++++++++++++++++++++++++
int openport_old(void){

  fd = open(USBPORT, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1)  {
    printf("init_serialport: Unable to open port\n");
    return 0;
  }

  if (tcgetattr(fd, &TermOpt) < 0) {
    printf("init_serialport: Couldn't get term attributes\n");
    return 0;
  }
  speed_t brate = BAUDRATE; // let you override switch below if needed

  cfsetispeed(&TermOpt, brate);
  cfsetospeed(&TermOpt, brate);

  // 8N1
  TermOpt.c_cflag &= ~PARENB;   // turn off parity
  TermOpt.c_cflag &= ~CSTOPB;   // Clear stop field, only one stop bit used in communication (most common). If set, two stop bits used in communication
  TermOpt.c_cflag &= ~CSIZE;
  TermOpt.c_cflag |= CS8;       //Set to 8 bits per byte

//    TermOpt.c_cflag &= ~IEXTEN;

//    TermOpt.c_cflag &= ~ICANON;  // turn off READ & ignore ctrl lines

//    TermOpt.c_iflag &= ~(IXON | IXOFF);

//    TermOpt.c_cc[VEOF]  = 0x1a;

    //TermOpt.c_cflag &= ~CRTSCTS;  // Disable RTS/CTS hardware flow control (most common)
//    TermOpt.c_cflag |= CRTSCTS;  // Enable RTS/CTS hardware flow control (most common)
//    TermOpt.c_cflag |= TIOCM_RTS;  // Enable RTS/CTS hardware flow control (most common)
//    TermOpt.c_cflag |= TIOCM_DTR;  // Enable DTR hardware flow control (most common)

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
//    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

//    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ECHONL | ISIG); // make raw
//    TermOpt.c_oflag &= ~OPOST; // make raw

//    TermOpt.c_lflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL); // Disable any special handling of received bytes

//    TermOpt.c_lflag &= ~OPOST; // Prevent special interpretation of output bytes (e.g. newline chars)
//    TermOpt.c_lflag &= ~ONLCR; // Prevent conversion of newline to carriage return/line feed
    //TermOpt.c_lflag &= ~ONOEOT; // Prevent removal of C-d chars (0x004) in output (NOT PRESENT IN LINUX)

//    TermOpt.c_oflag &= ~(OCRNL | ONLCR | ONLRET | ONOCR | OFILL | OLCUC | OPOST);

  TermOpt.c_cc[VMIN]  = 1;
  TermOpt.c_cc[VTIME] = 20;

  int RTS_flag, DTR_flag;
  RTS_flag = TIOCM_RTS;
  ioctl(fd,TIOCMBIS,&RTS_flag);//Set RTS pin
  DTR_flag = TIOCM_DTR;
  ioctl(fd,TIOCMBIS,&DTR_flag);//Set DTR pin


//   getchar();
//   ioctl(fd,TIOCMBIC,&RTS_flag);//Clear RTS pin
//   close(fd);

  if( tcsetattr(fd, TCSANOW, &TermOpt) != 0) {
    printf("init_serialport: Couldn't set term attributes\n");
    return 0;
  }


  return 1;
}
//------------------------ End openport --------------------------------

//++++++++++++++++++++++++ Messages to file ++++++++++++++++++++++++++++
void messages(char textLine[], int write){
  printf("%s\n", textLine);
  if(write){
    fwrite(textLine, strlen(textLine), 1, messageFile);
  }
}

//++++++++++++++++++++++++ Usage +++++++++++++++++++++++++++++++++++++++
static void usage(){
  printf("usage:\n");
  printf("engrave file=[*.bmp] <burntime=[1-100]> <laser=[1-100]>\n");
}

//++++++++++++++++++++++++ Return high byte ++++++++++++++++++++++++++++
unsigned char highByte(uint16_t number){
  unsigned char counter = 0;

  while(number > 255){
    counter++;
    number -= 256;
  }

  return counter;
}

//++++++++++++++++++++++++ Return low byte +++++++++++++++++++++++++++++
unsigned char lowByte(uint16_t number){
  unsigned char counter = 0;

  while(number > 255){
    counter++;
    number -= 256;
  }

  return number;
}


//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  struct winsize terminal;
  int i, j;
  int wrongArgs = 0;
  int fileOK = 0;
  int previewOK = 0;
  int laserPowerIdleOK = 1;
  int laserPowerIdle = 1;
  int previewWidth = 0;
  int previewHeight = 0;
  char argUPPER[1000];
  char argNumber[1000];
  int laser = 1, burntime = 1, laserIdle = 1;
  unsigned char tempUchar = 0;
  int isPause = 0;

  #define MAXWIDTH 490
  #define MAXHEIGHT 490

  uint16_t left, top;

  FILE *bmpFile, *fileCommands;
  char FileInfo[3];
  long FileSize;
  long LongTemp;
  char fileName[1000];
  uint32_t DataOffset;
  uint32_t HeaderSize;
  int32_t PictureWidth, PictureWidthBy8, FillBytes, BytesPerLine;
  int32_t PictureHeight;
  int reverseData = 0;
  int  IntTemp;
  uint16_t  ColorDepth;
  uint32_t CompressionType;
  uint32_t PictureSize;
  int32_t XPixelPerMeter;
  int32_t YPixelPerMeter;
  uint32_t ColorNumber;
  uint32_t ColorUsed;
  int PixelRed, PixelGreen, PixelBlue;
  int PixOK=0;
  char message[1000];

  FileInfo[2]='\0';

  // Command structure of NeJe KZ3000
  // FF AA 0B 03 01 64 0A 01 00 00 55
  //                |  |  |
  //                |  |  Laser Power in idle mode
  //                |  Laser Power in burn mode (10%)
  //                Burn time in ms (100ms)
  unsigned char commands[] = {0xFF, 0xAA, 0x0B, 0x03, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x55};

  // Parameter structure of bitmaps
  // FF AA 16 04 02 01 50 00 4A 00 97 01 58 00 BB 00 00 1F 69 01 55 55
  //                     |     |     |     |     |     |     |     |
  //                      Left   Top |     Height       Data  Width
  //                                 |                  Bytes
  //                                 |
  //                                 Width in Pixel plus Fillbits (dividable by 8)

  unsigned char bmpParam[]={0xFF, 0xAA, 0x16, 0x04, 0x02, 0x01, 0x50, 0x00, 0xEB, 0x00, 0xEB, 0x00, 0x10, 0x00, 0x10, 0x00, 0x00, 0x00, 0x20, 0x00, 0x10, 0x55};


  // Drive laser on edges of bitmap
  // FF AA 10 05 01 50 02 00 4A 00 97 01 55 00 BB 55
  //                     |     |     |     |    |
  //                      Left  Top   Width Height
  //
  unsigned char driveEdges[] = {0xFF, 0xAA, 0x10, 0x05, 0x01, 0x50, 0x02, 0x00, 0x4A, 0x00, 0x97, 0x01, 0x55, 0x00, 0xBB, 0x55};
  unsigned char  stopEdges[] = {0xFF, 0xAA, 0x10, 0x05, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55};


  unsigned char pauseMachine[]={0xFF, 0xAA, 0x08, 0x02, 0x01, 0x01, 0x00, 0x55};
  unsigned char resumeMachine[]={0xFF, 0xAA, 0x08, 0x02, 0x01, 0x01, 0x01, 0x55};
  unsigned char stopMachine[]={0xFF, 0xAA, 0x08, 0x02, 0x01, 0x01, 0x02, 0x55};


  // Bytes sent to initialize Neje device
  char init_1[] = {0xff, 0x09, 0x5a, 0xa5};
  char init_2[] = {0xff, 0xaa, 0x08, 0x01, 0x01, 0x5a, 0xa5, 0x55};
  char pic_data[]={0xff, 0xff, 0x7f, 0xfe, 0x3f, 0xfc, 0x1f, 0xf8, 0x0f, 0xf0, 0x07, 0xe0, 0x03, 0xc0, 0x01, 0x80, 0x01, 0x00, 0x00, 0x40, 0x04, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x04, 0x40, 0x00, 0x00, 0x01};


  system("cp IndicationPicture.bmp MachineBUSY.bmp");
  // Delete command files
  system("rm MachineSTOP.txt");
  system("rm MachinePAUSE.txt");
  system("rm MachineRESUME.txt");

  // Open messages file
  if((messageFile=fopen("Messages.txt","wb"))==NULL){
    printf("Can't open messages file!\n");
    exit(1);
  }

  // Check arguments
  for(i=0; i< argc; i++){
    if(strlen(argv[i]) > 200){
      wrongArgs = 1;
      sprintf(message, "Argument \"%s\" too long!\n", argv[i]);
      messages(message, 1);
    }
    else{
      strcpy(argUPPER, argv[i]);
      for(j=0; j<strlen(argUPPER); j++){
        argUPPER[j] = toupper(argUPPER[j]);
      }

      //Check for preview mode
      if(strstr(argUPPER, "PREVIEW") != NULL){
        previewOK = 1;
        laserPowerIdleOK = 0;
      }
      //Check preview width
      if(strstr(argUPPER, "PREVIEWWIDTH=") != NULL){
        strcpy(argNumber, argUPPER+13);
        //printf("argv[i]=%s, argNumber=%s\n", argv[i], argNumber);
        previewWidth = atoi(argNumber);
        if(previewWidth < 1 || previewWidth > MAXWIDTH){
          sprintf(message, "previewWidth %d out of range (1 to 490)!\n", previewWidth);
          messages(message, 1);
          wrongArgs = 1;
        }
        else{
          sprintf(message, "Setting previewWidth to %d\n", previewWidth);
          messages(message, 0);
          driveEdges[11]=highByte((uint16_t)(previewWidth));
          driveEdges[12]=lowByte((uint16_t)(previewWidth));
          //Left space
          driveEdges[7]=highByte((uint16_t)((MAXWIDTH - previewWidth)/2));
          driveEdges[8]= lowByte((uint16_t)((MAXWIDTH - previewWidth)/2));
        }
      }
      //Check preview height
      if(strstr(argUPPER, "PREVIEWHEIGHT=") != NULL){
        strcpy(argNumber, argUPPER+14);
        //printf("argv[i]=%s, argNumber=%s\n", argv[i], argNumber);
        previewHeight = atoi(argNumber);
        if(previewHeight < 1 || previewHeight > MAXHEIGHT){
          sprintf(message, "previewHeight %d out of range (1 to 490)!\n", previewHeight);
          messages(message, 1);
          wrongArgs = 1;
        }
        else{
          sprintf(message, "Setting previewHeight to %d\n", previewHeight);
          messages(message, 0);
          driveEdges[13]=highByte((uint16_t)(previewHeight));
          driveEdges[14]=lowByte((uint16_t)(previewHeight));
          //Top space
          driveEdges[9]=highByte((uint16_t)((MAXHEIGHT - previewHeight)/2));
          driveEdges[10]= lowByte((uint16_t)((MAXHEIGHT - previewHeight)/2));
        }
      }

      //Check laser power in idle mode
      if(strstr(argUPPER, "LASERPOWERIDLE=") != NULL){
        strcpy(argNumber, argUPPER+15);
        //printf("argv[i]=%s, argNumber=%s\n", argv[i], argNumber);
        laserPowerIdle = atoi(argNumber);
        if(laserPowerIdle < 1 || laserPowerIdle > 10){
          sprintf(message, "Laser power in idle mode %d out of range (1 to 10)!\n", laserPowerIdle);
          messages(message, 1);
          wrongArgs = 1;
        }
        else{
          sprintf(message, "Setting laser power in idle mode to %d\n", laserPowerIdle);
          messages(message, 0);
          tempUchar = (unsigned char)(laserPowerIdle);
          commands[7] = tempUchar;
        }
      }

      //Check laser power
      if(strstr(argUPPER, "LASER=") != NULL){
        laserPowerIdleOK = 0;
        strcpy(argNumber, argUPPER+6);
        //printf("argv[i]=%s, argNumber=%s\n", argv[i], argNumber);
        laser = atoi(argNumber);
        if(laser < 1 || laser > 100){
          sprintf(message, "Laser power %d out of range (1 to 100)!\n", laser);
          messages(message, 1);
          wrongArgs = 1;
        }
        else{
          sprintf(message, "Setting laser power to %d\n", laser);
          messages(message, 0);
          tempUchar = (unsigned char)(laser);
          commands[6] = tempUchar;
        }
      }
      //Check burntime
      if(strstr(argUPPER, "BURNTIME=") != NULL){
        laserPowerIdleOK = 0;
        strcpy(argNumber, argUPPER+9);
        //printf("argv[i]=%s, argNumber=%s\n", argv[i], argNumber);
        burntime = atoi(argNumber);
        if(burntime < 1 || burntime > 100){
          sprintf(message, "Burntime %d out of range (1 to 100)!\n", burntime);
          messages(message, 1);
          wrongArgs = 1;
        }
        else{
          sprintf(message, "Setting burntime to %d\n", burntime);
          messages(message, 0);
          tempUchar = (unsigned char)(burntime);
          commands[5] = tempUchar;
        }
      }
      //Check bitmap file
      if(strstr(argUPPER, "FILE=") != NULL){
        laserPowerIdleOK = 0;
        fileOK = 1;
        strcpy(fileName, argv[i]+5);
        sprintf(message, "Checking file '%s'\n", fileName);
        messages(message, 0);
        if((bmpFile=fopen(fileName,"rb"))==NULL){
          sprintf(message, "Can't open bitmap file '%s'!\n", argv[i]);
          messages(message, 1);
          fileOK = 0;
        }
        else{
          fread(&FileInfo, 2, 1, bmpFile);
          fread(&FileSize, 4, 1, bmpFile);
          fread(&LongTemp, 4, 1, bmpFile);
          fread(&DataOffset, 4, 1, bmpFile);
          fread(&HeaderSize, 4, 1, bmpFile);
          fread(&PictureWidth, 4, 1, bmpFile);
          fread(&PictureHeight, 4, 1, bmpFile);
          if(PictureHeight<0){
            PictureHeight = -1*PictureHeight;
            reverseData = 1;
          }
          fread(&IntTemp, 2, 1, bmpFile);
          fread(&ColorDepth, 2, 1, bmpFile);
          fread(&CompressionType, 4, 1, bmpFile);
          fread(&PictureSize, 4, 1, bmpFile);
          fread(&XPixelPerMeter, 4, 1, bmpFile);
          fread(&YPixelPerMeter, 4, 1, bmpFile);
          fread(&ColorNumber, 4, 1, bmpFile);
          fread(&ColorUsed, 4, 1, bmpFile);

          printf("FileInfo=%s\n", FileInfo);
          printf("FileSize=%ld\n", FileSize);
          printf("LongTemp=%ld\n", LongTemp);
          printf("DataOffset=%u\n", DataOffset);
          printf("HeaderSize=%u\n", HeaderSize);
          printf("PictureWidth=%u, Left=%u\n", PictureWidth, (uint16_t)((MAXWIDTH - PictureWidth)/2));
          printf("PictureHeight=%u, Top=%u\n", PictureHeight, (uint16_t)((MAXHEIGHT - PictureHeight)/2));
          printf("IntTemp=%d\n", IntTemp);
          printf("ColorDepth=%d\n", ColorDepth);
          printf("CompressionType=%u\n", CompressionType);
          printf("PictureSize=%u\n", PictureSize);
          printf("XPixelPerMeter=%u\n", XPixelPerMeter);
          printf("YPixelPerMeter=%u\n", YPixelPerMeter);
          printf("ColorNumber=%u\n", ColorNumber);
          printf("ColorUsed=%u\n", ColorUsed);

          if(FileInfo[0] != 'B' || FileInfo[1] != 'M'){
            sprintf(message, "Wrong Fileinfo (BM)!\n");
            messages(message, 1);
            fileOK = 0;
          }
          if(ColorDepth != 1){
            sprintf(message, "Wrong ColorDepth: %u (must be 2)!\n", ColorDepth);
            messages(message, 1);
            fileOK = 0;
          }
          if(CompressionType != 0){
            sprintf(message, "Wrong CompressionType (0)!\n");
            messages(message, 1);
            fileOK = 0;
          }

          if(PictureHeight > MAXHEIGHT){
            sprintf(message, "Picture height %u too large (max. 490)!\n", PictureHeight);
            messages(message, 1);
            fileOK = 0;
          }
          else{
            bmpParam[13]=highByte((uint16_t)(PictureHeight));
            bmpParam[14]=lowByte((uint16_t)(PictureHeight));
            // Center picture
            bmpParam[9]=highByte((uint16_t)((MAXHEIGHT - PictureHeight)/2));
            bmpParam[10]=lowByte((uint16_t)((MAXHEIGHT - PictureHeight)/2));

          }
          if(PictureHeight > MAXWIDTH){
            sprintf(message, "Picture width %u too large (max. 490)!\n", PictureWidth);
            messages(message, 1);
            fileOK = 0;
          }
          else{
            bmpParam[19]=highByte((uint16_t)(PictureWidth));
            bmpParam[20]=lowByte((uint16_t)(PictureWidth));
            bmpParam[7]=highByte((uint16_t)((MAXWIDTH - PictureWidth)/2));
            bmpParam[8]=lowByte((uint16_t)((MAXWIDTH - PictureWidth)/2));
            //Calculate width + fillbits, dividable by 8
            PictureWidthBy8 = PictureWidth;
            while(PictureWidthBy8 % 8 != 0){
              PictureWidthBy8++;
            }
            bmpParam[11]=highByte((uint16_t)(PictureWidthBy8));
            bmpParam[12]=lowByte((uint16_t)(PictureWidthBy8));

          }
          //Size of picture data in bytes
          bmpParam[17]=highByte((uint16_t)(PictureWidthBy8*PictureHeight/8));
          bmpParam[18]=lowByte((uint16_t)(PictureWidthBy8*PictureHeight/8));



          fclose(bmpFile);
        }
      }
    }
  }


  //Exit program if parameters wrong
  if((argc < 1 || wrongArgs != 0 || fileOK == 0) && !previewOK && !laserPowerIdleOK){
    usage();
    return(1);
  }

//  printf("bmpParam=");
//  for(i=0; i<22;i++){
//    printf("%02x, ", bmpParam[i]);
//  }
//  printf("\n");

  if(openport() == 0){
    sprintf(message, "Failed to open USB port!\n");
    messages(message, 1);
    exit(1);
  }

  // Initialize Neje device
  printf("\nWriting init_1: ");
  for(i=0; i<sizeof(init_1); i++){
    sendport(init_1[i]);
  }
  sleep(1);
  printf("\nRead:  ");
  while(readport()>-1);
  printf("\n");

  printf("\nWriting init_2: ");
  for(i=0; i<sizeof(init_2); i++){
    sendport(init_2[i]);
  }
  sleep(1);
  printf("\nRead:  ");
  while(readport()>-1);
  printf("\n");


  //Drive laser to preview edges and exit
  if(previewOK == 1){
    if(previewWidth > 0 && previewHeight > 0){
      // Send preview command line
      printf("\nWriting preview command: ");
      for(i=0; i<sizeof(driveEdges); i++){
        sendport(driveEdges[i]);
      }
      sleep(1);
      printf("\nRead:  ");
      while(readport()>-1);
      printf("\n");
    }
    else{
      // Stop preview
      printf("\nStopping preview command: ");
      for(i=0; i<sizeof(stopEdges); i++){
        sendport(stopEdges[i]);
      }
      sleep(1);
      printf("\nRead:  ");
      while(readport()>-1);
      printf("\n");
    }
    exit(1);
  }

  //Set idle laser power and exit
  if(laserPowerIdleOK){
    if(laserPowerIdle > 0 && laserPowerIdle < 11){
      // Send idle laser power command line
      commands[5] = (unsigned char)(1); //Burntime to 1
      commands[6] = (unsigned char)(1); //Laser to 1
      printf("\nWriting idle laser power command: ");
      for(i=0; i<sizeof(commands); i++){
        sendport(commands[i]);
      }
      sleep(1);
      printf("\nRead:  ");
      while(readport()>-1);
      printf("\n");
    }
    exit(1);
  }


  // Send command line (laser power, burn time)
  printf("\nWriting command: ");
  for(i=0; i<sizeof(commands); i++){
    sendport(commands[i]);
  }
  sleep(1);
  printf("\nRead:  ");
  while(readport()>-1);
  printf("\n");

  // Send bitmap header (width, height, data size)
  printf("\nWriting bitmap header: ");
  for(i=0; i<sizeof(bmpParam); i++){
    sendport(bmpParam[i]);
  }
  sleep(1);
  printf("\nRead:  ");
  while(readport()>-1);
  printf("\n");



  FillBytes = PictureWidthBy8/8;
  while(FillBytes % 4 !=0){
    FillBytes++;
  }
  FillBytes -= PictureWidthBy8/8;
  BytesPerLine = PictureWidthBy8/8 + FillBytes;

  printf("\nFillBytes=%d, PictureWidthBy8=%d\n", FillBytes, PictureWidthBy8);

  // Send bitmap data
  if((bmpFile=fopen(fileName,"rb"))==NULL){
    sprintf(message, "Can't open bitmap file '%s'!\n", argv[i]);
    messages(message, 1);
    fileOK = 0;
  }
  else{
    fseek(bmpFile, DataOffset, SEEK_SET);
    unsigned char bitmapLine[1000];
    unsigned char inverseBitmapLine[1000];
    printf("\nBitmap bytes:\n");
    int lineCount = 0;
    int reverseLine = 0;
    long byteDataCount = 0L;

    while(lineCount<PictureHeight){
      byteDataCount += BytesPerLine-FillBytes;
      lineCount++;

      if(reverseData == 0){//Send bitmap data from end to beginning
        fseek(bmpFile, -lineCount*BytesPerLine, SEEK_END);
      }
      fread(&bitmapLine, 1, BytesPerLine, bmpFile);
      printf("\n");
      if(reverseLine==0){
        reverseLine=1;
      }
      else{
        //Inverte line bytes
        unsigned char a;
        for(i=0; i<PictureWidthBy8/8; i++){
          a=bitmapLine[PictureWidthBy8/8-1-i];
          inverseBitmapLine[i] = 0;
          if(a & 128){
            inverseBitmapLine[i] += 1;
          }
          if(a & 64){
            inverseBitmapLine[i] += 2;
          }
          if(a & 32){
            inverseBitmapLine[i] += 4;
          }
          if(a & 16){
            inverseBitmapLine[i] += 8;
          }
          if(a & 8){
            inverseBitmapLine[i] += 16;
          }
          if(a & 4){
            inverseBitmapLine[i] += 32;
          }
          if(a & 2){
            inverseBitmapLine[i] += 64;
          }
          if(a & 1){
            inverseBitmapLine[i] += 128;
          }
        }
        for(i=0; i<PictureWidthBy8/8; i++){
          bitmapLine[i]=inverseBitmapLine[i];
        }
        reverseLine=0;
      }

      for(i=0; i<PictureWidthBy8/8; i++){
        //printf("%02x, ", a);
        sendport(bitmapLine[i]);
        /*
                                  // Check image print
                                  for(int j=7; j>-1; j--){
                                    if((1 << j) & bitmapLine[i]){
                                      printf("X");
                                    }
                                    else{
                                      printf(" ");
                                    }
                                  }
        */
      }
//      printf("|\n");
    }
    fclose(bmpFile);
    printf("\n\n%ld Bytes processed\n", byteDataCount);
  }



  int busy = -100;
  int a = -1;
  unsigned char received[]={0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  //Wait for transmission of laser temperature for x times (indicates job is finished)
  while(busy < 5){

    a = readport();
    if(a > -1){
      for(i=0; i<9; i++){
        received[i] = received[i+1];
      }
      received[9]=(unsigned char)(a);
      if(received[0]==0xFF && received[1]==0xAA){
        if(received[2]==0x0B && received[3]==0x0B && received[4]==0x02){
          busy++;
          printf("\nreceived sequence FF AA 0B 0B 02\n");
        }
        if(received[2]==0x0E){
          printf("Resetting busy variable.\n");
          busy = 0;
        }
      }
    }
    if(isPause == 1){
      busy = 0;
    }

    //Pause machine
    if((fileCommands=fopen("MachinePAUSE.txt","rb"))!=NULL){
      fclose(fileCommands);
      printf("\nPausing machine: ");
      for(i=0; i<sizeof(pauseMachine); i++){
        sendport(pauseMachine[i]);
      }
      printf("\n");
      system("rm MachinePAUSE.txt");
      isPause = 1;
    }

    //Resume job
    if((fileCommands=fopen("MachineRESUME.txt","rb"))!=NULL){
      fclose(fileCommands);
      printf("\nresuming job: ");
      for(i=0; i<sizeof(resumeMachine); i++){
        sendport(resumeMachine[i]);
      }
      printf("\n");
      system("rm MachineRESUME.txt");
      isPause = 0;
    }

    //Cancel job
    if((fileCommands=fopen("MachineSTOP.txt","rb"))!=NULL){
      fclose(fileCommands);
      printf("\nStop machine: ");
      for(i=0; i<sizeof(stopMachine); i++){
        sendport(stopMachine[i]);
      }
      printf("\n");
      system("rm MachineSTOP.txt");
      busy = 1000;
    }

  }

  system("rm MachineBUSY.bmp");

  close(fd);
  fclose(messageFile);
  return 0;
}
